import net.mikekohn.java_grinder.CPU;
import net.mikekohn.java_grinder.IOPort0;
import net.mikekohn.java_grinder.Timer;
import net.mikekohn.java_grinder.TimerListener;

// 16,000,000 cycles * 0.000050 = 800 cycles
// 800 cycles = 50us
// 20 interrupts = 1ms
// 40 interrupts = 2ms
// 400 interrupts = 20ms
// 20,000 interrupts = 1s

public class Chicken implements TimerListener
{
  public static int interrupt_count = 0;
  public static int servo_count = 0;
  public static int servo0_curr_value = 20;
  public static int servo1_curr_value = 20;
  public static int servo2_curr_value = 20;
  public static int servo3_curr_value = 20;
  public static int servo0_new_value = 20;
  public static int servo1_new_value = 20;
  public static int servo2_new_value = 20;
  public static int servo3_new_value = 20;

  public static void waitOneSecond()
  {
    // Spinning for 20,000 interrupts is 1 second.
    interrupt_count = 0;
    while(interrupt_count != 20000);
  }

  public static void wait200ms()
  {
    // Spinning for 4,000 interrupts is 200 milliseconds.
    interrupt_count = 0;
    while(interrupt_count != 4000);
  }

  public static void main(String args)
  {
    CPU.setClock16();

    IOPort0.setPinsAsOutput(0xff);
    IOPort0.setPinsLow(0xff);

    // Setup timer interrupt (aka, Java listener).
    Timer.setInterval(800, 1);
    Timer.setListener(true);

    while(true)
    {
      // Turn eyes on.
      IOPort0.setPinsHigh(0x30);

      // Arms are down.
      servo2_new_value = 20;
      servo3_new_value = 40;

      // Legs are down.
      servo0_new_value = 20;
      servo1_new_value = 40;

      waitOneSecond();
      waitOneSecond();

      // Blink eye.
      IOPort0.setPinsLow(0x10);
      wait200ms();
      IOPort0.setPinsHigh(0x10);
      wait200ms();
      IOPort0.setPinsLow(0x10);
      wait200ms();
      IOPort0.setPinsHigh(0x10);

      // Raise right ARM.
      servo3_new_value = 20;

      waitOneSecond();
      waitOneSecond();

      // Raise left ARM.
      servo2_new_value = 40;

      waitOneSecond();

      // Legs are up.
      servo0_new_value = 40;
      servo1_new_value = 20;

      waitOneSecond();

      // Turn eyes off.
      IOPort0.setPinsLow(0x30);

      // Left leg down.
      servo0_new_value = 20;

      waitOneSecond();
    }
  }

  public void timerInterrupt()
  {
    interrupt_count++;
    servo_count++;

    // After 20ms, reset servo pulse (turn off since transistors reverse
    // logic.  Set servo pulses to the values for the next 20ms cycle).
    if (servo_count == 400)
    {
      IOPort0.setPinsLow(0x0f);
      servo_count = 0;
      servo0_curr_value = servo0_new_value;
      servo1_curr_value = servo1_new_value;
      servo2_curr_value = servo2_new_value;
      servo3_curr_value = servo3_new_value;
    }

    // Set servo pulse to 0 (reverse logic because of transistor).
    if (servo0_curr_value == servo_count) { IOPort0.setPinsHigh(0x01); }
    if (servo1_curr_value == servo_count) { IOPort0.setPinsHigh(0x02); }
    if (servo2_curr_value == servo_count) { IOPort0.setPinsHigh(0x04); }
    if (servo3_curr_value == servo_count) { IOPort0.setPinsHigh(0x08); }
  }
}

